# Class to store global configuration variables
from __future__ import absolute_import
from pathlib import Path,PurePath
import os
import sys
from os import path
#sys.path.append(path.dirname(path.dirname(path.dirname(path.abspath(__file__)))))
#CONFIG_FILE=os.path.join(sys.path[-1],"streams_cca.yaml")
env_conf = os.getenv("HARMONCONF_YML")
if env_conf is None:
    CONFIG_FILE="streams.yaml"
else:
    CONFIG_FILE=env_conf

#print(f"CONFIG: using {CONFIG_FILE} for input")
class streams_info():      # for database globals
    def __init__(self, yaml_file = CONFIG_FILE):
        self.yaml_file = yaml_file
        self.yaml = self.read_yaml()

    def read_yaml(self):
        import yaml
        with open(self.yaml_file, "r") as f:
            return yaml.safe_load(f)

global harmonconf
harmonconf = streams_info()

def init_config():
    '''
    Initializes a ConfigParser instance
    @return No return value
    '''
    global cp
    cp=ConfigParser.RawConfigParser()

if __name__ == '__main__':
    init_config()



# Class that defines the vfld data from an Harmonie-lite model like:
# tasii
# sgl40h11
# nuuk750
# qaan40h11 (runs at 00, 06, 12, 18)
#
import logging
import pandas as pd
import os
import sys
import glob
import fileinput
import datetime
import numpy as np
from collections import OrderedDict
import csv
import subprocess
import re
import csv
import logging

from typing import Tuple

logger = logging.getLogger(__name__)

class vfld(object):
    def __init__(self,  model=None, period=None, 
                 flen=None, datadir=None, stream='CARRA'):
        self.stream = stream
        self.model = model
        self.period = period.split('-')
        self.flen= flen
        self.fhours = list(range(0,flen))
        self.datadir=datadir
        ifiles_model,dates_model = self._locate_files(self.datadir,self.model,self.period,self.flen)
        self.ifiles_model=ifiles_model
        self.dates = dates_model
        #self.dates=self._get_dates_from_files(self.ifiles_model)
        data_synop, data_temp, accum_synop = self._get_data(self.ifiles_model)
        self.data_synop = data_synop
        self.data_temp = data_temp
        self.temp_stations = self._get_temp_station_list(self.data_temp)
        self.accum_synop = accum_synop

    def _get_temp_station_list(self,data_temp) -> dict:
        ''' Extract the temp stations from the first row of data
            of the temp data section of the vfld file,
            which contains: stationId lat lon height
        '''
        # check if temp data is actually there
        #
        tempStations=OrderedDict()
        for date in data_temp.keys():
            if isinstance(data_temp[date], pd.DataFrame):
                tempStations[date] = [data_temp[date].iloc[0][0],data_temp[date].iloc[0][1],
                                      data_temp[date].iloc[0][2], data_temp[date].iloc[0][3]]
            else:
                tempStations[date] = [np.nan,np.nan,np.nan,np.nan]
        return tempStations

    def _get_data(self,ifiles): -> Tuple[dict,dict,dict]
    # do something
        '''
        Extract the data from all ifiles in a df for SYNOP and TEMP data
        '''
        data_synop=OrderedDict()
        data_temp=OrderedDict()
        accum_synop =OrderedDict()
        for i,ifile in enumerate(ifiles):
            date=self.dates[i]
            data_synop[date], data_temp[date], accum_synop[date] = self._split_data(ifile)
            # print a warning if synop data is not there:
            # TODO: if no synop, don't include model!
            if len(data_synop[date]) == 0:
                logger.info("WARNING: synop data for %s[%s] is empty!"%(self.model,date))
                data_synop[date] = 'None'
            if len(data_temp[date]) == 0:
                logger.info("WARNING: temp data for %s[%s] is empty!"%(self.model,date))
                data_temp[date] = 'None'
        return data_synop, data_temp, accum_synop

    def _dtg_expected_carra(self,flen,dates) -> list:
        '''
           return dates expected to be found in the carra streams
        '''
        #init times: 00 and 12. Forecast hours expected from 0-30. Every 1h until 6, then every 3 h
        #init times: 03,06,09,15,18,21. Forecast hours expected from 0-3. Every 1h.
        fhours_long=[str(i).zfill(2) for i in range(0,7)] + [str(i).zfill(2) for i in range(9,flen,3)]
        fhours_short=[str(i).zfill(2) for i in range(0,4)]
        init_expected = [str(i).zfill(2) for i in range(0,22,3)]
        dtg_expected=[]
        for date in dates:
            for init in init_expected:
                if init in ['00', '12']:
                    for hour in fhours_long:
                        dtg_expected.append(''.join([date,init,str(hour).zfill(2)]))
                else:
                    for hour in fhours_short:
                        dtg_expected.append(''.join([date,init,str(hour).zfill(2)]))
        return dtg_expected

    def _dtg_expected_dmi(self,model,flen,dates) -> list:
        '''
           return dates expected to be found in the carra streams
        '''
        fhours=[str(i).zfill(2) for i in range(0,flen+1)]
        init_expected = [str(i).zfill(2) for i in range(0,22,3)]
        init_reduced = ['03','09','15','21']
        special_models=['tasii','db_ondemand','nk_ondemand']
        dtg_expected=[]
        for date in dates:
            if model in special_models:
                logger.info("Using tasii-like model %s for calculating expected dtgs"%model)
                for init in init_reduced:
                    for hour in fhours:
                        dtg_expected.append(''.join([date,init,str(hour).zfill(2)]))
            else:    
                logger.info("Using standard on-demand model %s for calculating expected dtgs"%model)
                for init in init_expected:
                    for hour in fhours:
                        dtg_expected.append(''.join([date,init,str(hour).zfill(2)]))
        return dtg_expected

    def _locate_files(self,datadir,model,period,flen):
        '''
        Locate the files to process from each model.
        period = YYYYMMDD_beg-YYYYMMDD_end
        Shift the file name by -3 h if the model is tasii
        '''
        date_ini=datetime.datetime.strptime(period[0],'%Y%m%d')
        date_end=datetime.datetime.strptime(period[1],'%Y%m%d')
        dates = [date_ini + datetime.timedelta(days=x) for x in range(0, (date_end-date_ini).days + 1)]
        model_dates=[datetime.datetime.strftime(date,'%Y%m%d') for date in dates]
        ifiles_model = []
        if self.stream=='CARRA':
            dtgs=self._dtg_expected_carra(flen,model_dates)
        elif self.stream=='DMI':
            dtgs=self._dtg_expected_dmi(model,flen,model_dates)
        else:
            logger.error("Stream %s unknown!"%self.stream)
            logger.error("Exiting program...")
            sys.exit()
        for dtg in dtgs:
            fname=''.join(['vfld',model,dtg])
            fdir='/'.join([datadir,model])
            ifile=os.path.join(fdir,fname)
            if os.path.exists(ifile):
                ifiles_model.append(ifile)
            else:
                ifiles_model.append('None')
        if (len(ifiles_model) == 0) or (len(set(ifiles_model)) == 1): # if all elements equal all None!
            logger.info("WARNING: no %s data found for dates %s"%(model,model_dates))
            logger.info(ifiles_model)
        logger.debug("first file for model %s: %s"%(self.model,ifiles_model[0]))
        logger.debug("last file for model %s: %s"%(self.model,ifiles_model[-1]))
        return ifiles_model, dtgs

    def _get_synop_vars(self,ifile):
        '''
        Extract information about the SYNOP variables in vfld file
        '''
        #Read this file to determine number of variables in file:
        first_two=[]
        try:
            with open(ifile,'r') as f:
                logger.debug("File %s readable"%ifile)
        except IOError:
            logger.error("ERROR: File %s exists, but I cannot read it!"%ifile)
            logger.error("I am out!")
            logger.error("TODO: Fix this")
            sys.exit()
        with open(ifile,'r') as f:
            first_two.extend(str(f.readline()).rstrip() for i in range(2))
        nsynop_vars=int(first_two[-1]) # number of variables in synop data
        ntemp_stations=int(first_two[-1]) # number of temp stations in file
        nsynop_stations,ntemp_stations,ver_file =  first_two[0].split() # number of synop stations in file
        nsynop_stations=int(nsynop_stations)
        ntemp_stations = int(ntemp_stations)
    
        lines_header =[]
        with open(ifile) as f:
                lines_header.extend(f.readline().rstrip() for i in range(nsynop_vars+2))
        lines_clean =  [re.sub('\s+',' ',i).strip() for i in lines_header]        
        vars_synop=[i.split(' ')[0] for i in lines_clean[2:]]
        accum_synop=[i.split(' ')[1] for i in lines_clean[2:]] #accumulation times. Needed to write the data at the end
        if 'FI' in vars_synop:
            colnames= ['stationId','lat','lon'] + vars_synop
            start_col_replace = 3
        else:
            colnames=['stationId','lat','lon','HH'] + vars_synop
            start_col_replace = 4
        ignore_rows = 2 + nsynop_vars # number of rows to ignore before reading the actual synop data
        return colnames, nsynop_stations, ignore_rows, ntemp_stations, accum_synop

    def _split_data(self,ifile):
        '''
        Split the data from ifile into SYNOP and TEMP
        in two data frames
        '''
        cols_temp = ['PP','FI','TT','RH','DD','FF','QQ','TD'] #this set of temp variables is constant
        header_temp=OrderedDict()
        if ifile != 'None':
            colnames, nsynop_stations, ignore_rows, ntemp_stations, accum_synop = self._get_synop_vars(ifile)
            data_synop = pd.read_csv(ifile,sep=r"\s+",engine='python',header=None,index_col=None,
                    dtype=str,skiprows=ignore_rows,nrows=nsynop_stations)
            #NOTE: will name the columns as VariableName+AccumulationTime. To be split afterwards 
            #when writing the data in vfld format for monitor in vfld_monitor
            #data_synop.columns=colnames
            if 'FI' in colnames:
                data_synop.columns=colnames[0:3]+[' '.join(str(i) for i in col) for col in zip(colnames[3:],accum_synop)]
            else:
                data_synop.columns=colnames[0:4]+[' '.join(str(i) for i in col) for col in zip(colnames[4:],accum_synop)]
            ignore_temp=ignore_rows+data_synop.shape[0]+10
            data_temp =  pd.read_csv(ifile,sep=r"\s+",engine='python',header=None,index_col=None,names=cols_temp,
                                      dtype=str,skiprows=ignore_temp)
        else:
            data_synop = 'None'
            data_temp = 'None'
            accum_synop = 'None'
        return data_synop, data_temp, accum_synop

class vfld_monitor(object):
    '''
        Take dataframe with vfld data and format the
        rows and columns to write the whole thing to 
        a standard vfld ascii file that monitor can read
    '''

    def __init__(self, model=None, date=None,df_synop=None, df_temp=None, outdir=None):
        self.model = model #model to name the vfld file
        self.date=date
        self.df_synop = df_synop # pandas dataframe with synop data
        self.df_temp = df_temp # pandas dataframe with temp data
        self.outdir = outdir
        self.synop_cols = self.df_synop.columns
        self.df_out = self._format_data(df_synop,df_temp)
        
    def _format_data(self,df_synop,df_temp):
        '''
        Format the data to write in monitor vfld format.
        First create a dummy set of column names with the 
        same length as the length of the synop data.
        Then put below the info about
        ns_synop ns_temp version_4
        number_of_vars_synop
        list of vars from synop
        synopdata
        header temp
        temp data
        '''
        colst = df_temp.columns
        colss = df_synop.columns
        ns_synop = df_synop.shape[0]
        #to figure out the number of stations in the concat temp dataframe,
        # get the rows with index == 0, since only on the first line of the 
        # original temp dataframes this information existed
        # Each new set of data will start at 0. 
        # NOTE: This seems to work only with non overlappping data sets!
        #ns_temp = len(df_temp[df_temp.index.values == 0])

        #alternative: since I read all in string format, only the 
        #strings not containing a . will be station names in the PP column
        temp_stations_PP=[st for st in df_temp['PP'].values if '.' not in st]
        ns_temp = len(temp_stations_PP)

        #declaring these values for header as strings is the only way I can ensure these numbers are written as non-float
        header_synop=[str(ns_synop), str(ns_temp), str(4)]
        df_out=pd.DataFrame(columns=colss) 
        if 'FI 0' in colss:
            nvars_synop = df_synop.shape[1]-3 # subtract: stationId,lat,lon
            varlist_synop=colss[3:]  
            nvars=len(colss[3:])
        else:
            nvars_synop = df_synop.shape[1]-4 #subtract stationId,lat,lon,hh
            varlist_synop=colss[4:]  
            nvars=len(colss[4:])
        #write first line of file:    
        df_out=df_out.append({'stationId':header_synop[0],'lat':header_synop[1],'lon':header_synop[2]},ignore_index=True)
        df_out=df_out.append({'lat':str(nvars)},ignore_index=True)
        for var in varlist_synop:
            df_out = df_out.append({'stationId':var},ignore_index=True)
        df_out = df_out.append(df_synop,ignore_index=True)    
        if self.model in ['carra','carra_beta1','carra_beta2','carra_rc1']:
            logger.debug("carra or carra branch (%s): Setting pressure levels to 10"%self.model)
            df_out = df_out.append({'stationId':str(10)},ignore_index=True) #10 pressure levels for carra (constant)
        else:
            df_out = df_out.append({'stationId':str(11)},ignore_index=True) #11 pressure levels (constant)
        df_out = df_out.append({'stationId':str(8)},ignore_index=True) #8 variables for temp profiles (constant)
        for var in colst:
            df_out = df_out.append({'stationId':var+' 0'},ignore_index=True) #include accumulation time for temp vars
        #fill_these = ['stationId', 'lat', 'lon', 'FI', 'NN', 'DD', 'FF', 'TT']
        fill_these = self.synop_cols[0:8]
        #df_temp = df_temp.fillna(value=pd.np.nan, inplace=True) #get rid of None?
        for i in enumerate(df_temp['PP'].values):
            collect_dict=OrderedDict()
            for k,col in enumerate(colst):
                collect_dict[fill_these[k]] = df_temp[col].values[i[0]]
            #for col in self.synop_cols[8:]:
            #    collect_dict[col] = ''   
            df_out = df_out.append(collect_dict,ignore_index=True)
        for col in df_out.columns:
            df_out[col].replace('None', '', inplace=True)
        return df_out


    def write_vfld(self):
        ofile=os.path.join(self.outdir,''.join(['vfld',self.model,self.date]))
        #the extra QUOTE_NONE is to avoid using extra "" in output for var names, and the escapechar 
        #so it won't complain about no escapechar unset
        self.df_out.to_csv(ofile,sep=' ',header=False,index=False,na_rep='',quoting=csv.QUOTE_NONE,quotechar='',escapechar=' ')


if __name__ == '__main__':
    models=[]
    model='tasii'
    period='20190601-20190601'
    flen=52
    datadir='/data/cap/code_development_hpc/scripts_verif/merge_scripts/merge_vfld/example_data'
    #datadir='/netapp/dmiusr/aldtst/vfld'
    tasii = vfld(model=model, period=period, flen=52, datadir=datadir)
    sgl40h11 = vfld(model='sgl40h11', period=period, flen=52, datadir=datadir)
    qaan40h11 = vfld(model='qaan40h11', period=period, flen=52, datadir=datadir)
    #models=[tasii, sgl40h11, nuuk750, qaan40h11]
    models=[tasii, sgl40h11, qaan40h11]
    date=tasii.dates[0]
    frames_synop = [f for f in models if isinstance(f.data_synop[date],pd.DataFrame)]
    frames_temp = [f for f in models if isinstance(f.data_temp[date],pd.DataFrame)]
    dfs=[f.data_synop[date] for f in frames_synop]
    dft=[f.data_temp[date] for f in frames_temp]
    df_synop = pd.concat(dfs,sort=False)
    df_temp = pd.concat(dft)
    mon_save= vfld_monitor(model='gl',date=date,df_synop=df_synop,df_temp=df_temp,outdir=datadir)
    mon_save.write_vfld()




#from hartools import vobs
from hartools import vobs as vobs
from pytest import mark
from pytest import fixture
import os
import sys
import subprocess
import shutil

@fixture
def prepare_data():
    """
    Extract data and return path
    """
    tarball = "vobs_sample.tgz"
    wrkdir = "/tmp/vobs"
    if not os.path.isdir(wrkdir): os.makedirs(wrkdir)
    cwd = os.getcwd()
    shutil.copy(os.path.join(cwd,tarball),wrkdir)
    os.chdir(wrkdir)
    cmd = f"tar -zxvf {tarball} --strip-components 6"
    try:
        out=subprocess.check_output(cmd,shell=True)
    except subprocess.CalledProcessError as err:
        print(f"Error in subprocess {err}")
    datapath = wrkdir
    return datapath

@fixture
def set_input(data_path):
    period = '20220115-20220115'
    datadir = data_path
    return [period, datadir]
    
    
def Testvobs(set_input):  
    period = set_input[0]
    datadir = set_input[1]
    #period='20220115-20220115'
    #datadir='/scratch/ms/dk/nhd/tmp/vobs/'
    vobs_data = vobs(period=period, datadir=datadir)
    print(vobs_data.data_synop)
    assert vobs_data.data_synop.shape[0] == 24




"""The vobs class"""
import logging
import datetime
import os, sys
import numpy as np
import pandas as pd
import re
from collections import OrderedDict
from typing import Tuple
logger = logging.getLogger(__name__)

class vobs:
    def __init__(self,period=None, datadir=None, source="asc") -> None:
        """Initialize the vobs class

        Args:
            period ([str], optional): The period to include. String separated by -. Defaults to None.
            datadir ([str], optional): The path with the data. Defaults to None.
            source (str, optional): The source type. Defaults to "asc". No 
        """
        self.flen = 24
        self.period = period.split('-')
        self.datadir = datadir
        self.ifiles,self.dates = self._locate_files(self.datadir,self.period,self.flen)
        self.ftimes = self._dates_expand(self.period,self.flen)
        data_synop, data_temp, accum_synop = self._get_data(self.ifiles)
        self.data_synop = data_synop
        self.data_temp = data_temp
        #self.temp_stations = self._get_temp_station_list(self.data_temp)
        self.accum_synop = accum_synop

    def _locate_files(self,datadir,period,flen):
        """Locate the files available in path datadir

        Args:
            datadir ([str]): The path with the vobs files
            period ([str]): The period in format of two strings with YYYYMMDD, separated by a 
                            single dash
            flen ([int]]): The number of hours. Defaults to 24

        Returns:
            [list]: Two lists with the files and the dates
        """
        
        date_ini=datetime.datetime.strptime(period[0],'%Y%m%d')
        date_end=datetime.datetime.strptime(period[1],'%Y%m%d')
        dates = [date_ini + datetime.timedelta(days=x) for x in range(0, (date_end-date_ini).days + 1)]
        str_dates=[datetime.datetime.strftime(date,'%Y%m%d') for date in dates]
        ifiles = []
        dtgs=[]

        for date in str_dates:
            for hour in range(0,flen):
                date_file = datetime.datetime.strptime(date,'%Y%m%d') + datetime.timedelta(seconds=3600*hour)
                date_file = datetime.datetime.strftime(date_file,'%Y%m%d%H')
                dtgs.append(date_file)
                fname=''.join(['vobs',date_file])
                ifile=os.path.join(datadir,fname)
                if os.path.exists(ifile):
                    ifiles.append(ifile)
                else:
                    ifiles.append('None')
        if (len(ifiles) == 0) or (len(set(ifiles)) == 1): # if all elements equal all None!
            logger.info("WARNING: no data found for dates %s"%(str_dates))
            logger.info(ifiles)
        logger.debug("first file: %s"%(ifiles[0]))
        logger.debug("last file: %s"%(ifiles[-1]))
        return ifiles, dtgs

    def _get_data(self,ifiles):
        '''
        Extract the data from all ifiles in a df for SYNOP and TEMP data
        '''
        data_synop=OrderedDict()
        data_temp=OrderedDict()
        accum_synop =OrderedDict()
        for i,ifile in enumerate(ifiles):
            #date=self.dates[i] #ORIGINAL: now storing in ftimes to match with vfld dates
            date=self.ftimes[i]
            data_synop[date], data_temp[date], accum_synop[date] = self._split_data(ifile)
            # print a warning if synop data is not there:
            # TODO: if no synop, don't include model!
            if len(data_synop[date]) == 0:
                logger.info("WARNING: no synop data for %s!"%(date))
                data_synop[date] = 'None'
            if len(data_temp[date]) == 0:
                logger.info("WARNING: no temp data for %s!"%(date))
                data_temp[date] = 'None'
        return data_synop, data_temp, accum_synop

    def read_synop(infile,varlist) -> pd.DataFrame:
        rawData=pd.read_csv(infile,delimiter=" ")
        if 'FI' in varlist:
            columns = ['stationId','lat','lon'] + varlist
            rawData.columns=columns
            rawData=rawData.rename(columns={'FI': 'HH'})
        else:    
            columns=['stationId','lat','lon','HH']+varlist
            rawData.columns=columns
        if 'PE' in varlist:
            rawData=rawData.rename(columns={'PE':'PE1'})
        return rawData

    def _split_data(self,ifile) -> Tuple[pd.DataFrame,pd.DataFrame,pd.DataFrame]:
        '''
        Split the data from ifile into SYNOP and TEMP
        in two data frames
        '''
        cols_temp = ['PP','FI','TT','RH','DD','FF','QQ','TD'] #this set of temp variables is constant
        header_temp=OrderedDict()
        print(f"Reading {ifile}")
        if ifile != 'None':
            colnames, nsynop_stations, ignore_rows, ntemp_stations, accum_synop = self._get_synop_vars(ifile)
            if nsynop_stations != 0:
                data_synop = pd.read_csv(ifile,sep=r"\s+",engine='python',header=None,index_col=None,
                    dtype=str,skiprows=ignore_rows,nrows=nsynop_stations)
                data_synop.columns=colnames
                nrows_synop = data_synop.shape[0]
            elif nsynop_stations == 0:
                print(f"No synop data: {nsynop_stations}")
                data_synop = "None" #pd.DataFrame(columns = colnames)
                accum_synop = "None"
                nrows_synop = 0
           
            if ntemp_stations != 0:
                #when writing the data in vfld format for monitor in vfld_monitor
                ignore_temp=ignore_rows+nrows_synop+10 #ignore first rows on this  part
                data_temp =  pd.read_csv(ifile,sep=r"\s+",engine='python',header=None,index_col=None,names=cols_temp,
                                      dtype=str,skiprows=ignore_temp)
            elif ntemp_stations == 0:
                print(f"No temp data: {ntemp_stations}")
                data_temp = "None" #pd.DataFrame(columns = cols_temp)
        else:
            data_synop = 'None'
            data_temp = 'None'
            accum_synop = 'None'
    
        return data_synop, data_temp, accum_synop
    
    def _get_synop_vars(self,ifile):
        '''
        Extract information about the SYNOP variables in vfld file
        '''
        #Read this file to determine number of variables in file:
        first_two=[]
        with open(ifile) as f:
            first_two.extend(str(f.readline()).rstrip() for i in range(2))
        nsynop_vars=int(first_two[-1]) # number of variables in synop data
        ntemp_stations=int(first_two[-1]) # number of temp stations in file
        nsynop_stations,ntemp_stations,ver_file =  first_two[0].split() # number of synop stations in file
        nsynop_stations=int(nsynop_stations)
        ntemp_stations = int(ntemp_stations)
    
        lines_header =[]
        with open(ifile) as f:
                lines_header.extend(f.readline().rstrip() for i in range(nsynop_vars+2))
        #this regex is to remove all extrs spaces and replace it with one
        #print(f"check lines before {lines_header}")        
        #Deprecated style, apparently:
        #lines_clean =  [re.sub('\s+',' ',i).strip() for i in lines_header]   
        lines_clean =  [re.sub(' +',' ',i).strip() for i in lines_header]   
        #print(f"check lines after {lines_clean}")     
        vars_synop=[i.split(' ')[0] for i in lines_clean[2:]]
        accum_synop=[i.split(' ')[1] for i in lines_clean[2:]] #accumulation times. Needed to write the data at the end
        if 'FI' in vars_synop:
            colnames= ['stationId','lat','lon'] + vars_synop
            start_col_replace = 3
        else:
            colnames=['stationId','lat','lon','HH'] + vars_synop
            start_col_replace = 4
        ignore_rows = 2 + nsynop_vars # number of rows to ignore before reading the actual synop data
        return colnames, nsynop_stations, ignore_rows, ntemp_stations, accum_synop

    def _dates_expand(self,period,flen) -> list:
        '''
        Expand the dates into YYYYMMDD format
        '''
        date_ini=datetime.datetime.strptime(period[0],'%Y%m%d')
        date_end=datetime.datetime.strptime(period[1],'%Y%m%d')
        dates = [date_ini + datetime.timedelta(days=x) for x in range(0, (date_end-date_ini).days + 1)]
        dtgs=[]
        str_dates=[datetime.datetime.strftime(date,'%Y%m%d') for date in dates]
        for date in str_dates:
            #print(date)
            for hour in range(0,flen):
                dtgs.append(''.join([date[0:8],str(hour).zfill(2)]))
        return dtgs            

    
    def read_temp(time,date):
        '''Read the temp stations. Not yet implemented'''
        pass
        
if __name__ == '__main__':
    period='20220115-20220115'
    datadir='/tmp/vobs/'
    vobs_data = vobs(period=period, datadir=datadir)
    print(vobs_data.data_synop)



"""Console script for hartools."""
import argparse
import sys

from datetime import date
from typing import Dict

from driconfig import DriConfig
from pydantic import BaseModel



def main():
    """Console script for hartools."""
    parser = argparse.ArgumentParser()
    #parser.add_argument('_', nargs='*')

    parser.add_argument('-yaml',metavar='config file in yaml format',
                        type=str,
                        default=None,
                        required=True)

    args = parser.parse_args()
    conf_file = args.yaml

    print("Arguments: " + str(args.yaml))
    #print("Arguments: " + str(args._))
    #print("Replace this message by putting your code into "
    #      "hartools.cli.main")
    #return 0
    #class DateInterval(BaseModel):
    #  """Model for the `date_interval` configuration."""
    #  start: date
    #  end: date
    #
    #
    #class AppConfig(DriConfig):
    #   """Interface for the config/config.yaml file."""
    #
    #   class Config:
    #       """Configure the YAML file location."""
    #       config_folder = "../examples"
    #       config_file_name = args.yaml
    #       model_parameters: Dict[str, float]
    #       #date_interval: DateInterval
    #   #print(Config.config_file_name)
    #config = AppConfig()
    #print(config.json(indent=4))
    #class Model(BaseModel):
    #  model: str

    #class AppConfig(DriConfig):
    #   """Interface for the config/config.yaml file."""
    #   class Config:
    #       """Configure the YAML file location."""
    #       config_folder = "."
    #       config_file_name = args.yaml
    #
    #   #model_parameters: Dict[str, float]
    #   #hartools_parameters: Dict[str,str,str]
    #   #hartools_parameters: Dict[str, float]
    #   model: Model
    #
    #config = AppConfig()
    #config = AppConfig(config_folder=".",config_file_name="config.yaml")
    #print(config.json(indent=4))
    #class Config:
    #    config_file_name = args.yaml
 
    #HarConfig = Config()

    class DateInterval(BaseModel):
      """Model for the `date_interval` configuration."""
      start: str
      end: str
    
    #ifile = "config_hartools.yaml"
    class HarConfig(DriConfig):
       """Interface for the config/config.yaml file."""
       class Config:
           """Configure the YAML file location."""
           config_folder = "."
           config_file_name = args.yaml
    
       model_data: Dict[str, str]
       date_interval: DateInterval
    
    HarToolsConf = HarConfig()
    print(HarToolsConf.json(indent=4))




if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover
    #main()  # pragma: no cover



"""Top-level package for hartools."""

__author__ = """Carlos Peralta"""
__email__ = 'carlos9917@gmail.com'
__version__ = '0.1.1'

from hartools.hario.vobs import vobs
#from hartools.harmon.config import config
from hartools.harmon.log_timestamps import log_timestamps




#!/usr/bin/env python3

'''
Some functions to extract data from the timestamps
for the throughput plots
'''

from datetime import datetime
from datetime import timedelta
import pandas as pd
#need this to use it in crontab
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import os
import matplotlib.dates as md
import sys
import re
import time
#Avoid pandas Future warning
from pandas.plotting import register_matplotlib_converters
register_matplotlib_converters()
from collections import OrderedDict

import sqlite3
from hartools.harmon.config import harmonconf as harmonconf
from dateutil import relativedelta
BUFFER_DAYS = 30 #number of days to calculate ideal and real speed

def logs_extract(data,project,stream,remove_years):
    '''
    Calculate min and max DTG in data
    cols in data: ['stream', 'logfile', 'timestamp', 'yyyymmdd', 'simdate']
    The DTGS are extracted  from the paths of the logfiles.tar
    '''
    if remove_years == "None":
        #use here ONLY if want to plot the whole thing from the beginning
        logs=data[data['stream']==stream].logfile.tolist() # check log files for this stream
        log_dates=[]
        for log in logs:
            yyyymmddhh ="".join([i for i in log.split("/") if i.isdigit()])
            log_dates.append(yyyymmddhh)
        DTG_max=str(max(log_dates))
        DTG_min,stuff=stream_periods(project,stream) 
    else:
        # extract logs for a given date
        # Get rid of the selected years TOChECK: can I provide several numbers?
        data["remove"]=data["Year/Month/Day"].str.contains(remove_years) 
        data=data[data['remove'] == False] #drop the selected years
        #check for the logs at the beginning of the year:
        current_year = datetime.strftime(datetime.now(),"%Y")
        beg_year = pd.to_datetime(datetime.strptime(current_year+"0101","%Y%m%d"))
        #NOTE: Pandas warning. I will turn off this shit for now
        import warnings
        from pandas.core.common import SettingWithCopyWarning
        warnings.simplefilter(action="ignore", category=SettingWithCopyWarning)

        data["times"]=list(pd.to_datetime(data["timestamp"]).values) #so i dont get fucking warning?
        logs=data[data['stream']==stream].logfile.tolist() # check log files for this stream
        log_dates=[]
        for log in logs:
            yyyymmddhh ="".join([i for i in log.split("/") if i.isdigit()])
            log_dates.append(yyyymmddhh)
        #print(f"{stream} and dates found {log_dates} ")
        DTG_max=str(max(log_dates))
        DTG_min=str(min(log_dates))
    return DTG_min,DTG_max

def proglog_user(stream):
    '''
    identify the progress log according to user
    '''
    pfile=os.path.join(*[rdir,stream,'progress.log'])

def test_months_stream(project,stream):
    #sanity check: is this stream finished?
    #Calculate expected number of months
    conf = harmonconf.yaml["PROJECTS"][project]["STREAMS"]
    str_comp,str_end = stream_completed(project,stream)
    if str_comp: print(f"{stream} finished on {str_end}")
    DTG_min,DTG_max = str(conf[stream]["BEG_DATE"]),str(conf[stream]["END_DATE"])
    date1 = datetime.strptime(DTG_min, '%Y%m%d%H')
    date2 = datetime.strptime(DTG_max, '%Y%m%d%H')
    difference = relativedelta.relativedelta(date2, date1)
    months = difference.months
    years = difference.years
    # add in the number of months (12) for difference in years
    months += 12 * difference.years
    print(f"EXPECTED number of months for {stream}:{months}")

def calc_total_months(project):
    '''
    Calculate the total number of months for a project   
    '''
    conf = harmonconf.yaml["PROJECTS"][project]["STREAMS"]
    months2sim = 0
    streams=[key for key in conf.keys()] # Doing it for all
    #ESPECIAL CASE for PANARCTIC
    for st in streams:
        #print("Calling it here?")
        #str_comp,str_end = stream_completed(project,st)
        #if str_comp: print(f"{st} finished on {str_end}")
        DTG_min,DTG_max = str(conf[st]["BEG_DATE"]),str(conf[st]["END_DATE"])
        date1 = datetime.strptime(DTG_min, '%Y%m%d%H')
        date2 = datetime.strptime(DTG_max, '%Y%m%d%H')
        difference = relativedelta.relativedelta(date2, date1)
        months = difference.months
        years = difference.years
        # add in the number of months (12) for difference in years
        months += 12 * difference.years
        months2sim += months
    #print(f"TOTAL EXPECTED number of months for {project} ({streams}):{months2sim}")
    return months2sim

    

def progress_bar(project):
    '''
    TODO
    read the sql file with the stream information
    and calculate current completion percentage.
    It uses this file since I am storing the information
    there.
    '''
    dbase=harmonconf.yaml["PROJECTS"][project]["DBASE"]
    months2sim = calc_total_months(project)
    conn=sqlite3.connect(dbase)
    sql_command = "SELECT * FROM daily_logs"
    data=pd.read_sql(sql_command, conn)
    #current streams running
    conf = harmonconf.yaml["PROJECTS"][project]["STREAMS"]
    #streams=[key for key in conf.keys() if conf[key]["ACTIVE"]]
    streams=[key for key in conf.keys()] # Doing it for all
    progress_stream={}
    months_total=0
    for st in streams:
        DTG_min,DTG_max = logs_extract(data,project,st,"None")
        #print(f"PROG: Min and max for {st}: {DTG_min},{DTG_max}")
        date1 = datetime.strptime(DTG_min, '%Y%m%d%H')
        date2 = datetime.strptime(DTG_max, '%Y%m%d%H')
        difference = relativedelta.relativedelta(date2, date1)
        months = difference.months
        years = difference.years
        # add in the number of months (12) for difference in years
        months += 12 * difference.years
        print(f"Total months simulated for {st}:{months}")
        #test_months_stream(project,st) #TO check number of months in stream
        months_total += months
    total = round(100*months_total/months2sim,2)
    months_total=0
    # This loop is only for printing out some extra info !!!
    for st in streams:
        DTG_min,DTG_max = logs_extract(data,project,st,"None")
        date1 = datetime.strptime(DTG_min, '%Y%m%d%H')
        date2 = datetime.strptime(DTG_max, '%Y%m%d%H')
        difference = relativedelta.relativedelta(date2, date1)
        months = difference.months
        years = difference.years
        # add in the number of months (12) for difference in years
        months += 12 * difference.years
        months_total += months
    print(f"Number of months completed: {months_total}")
    return total

def html_table_carra(rfile,project,stream,TOAstream,speed_30d,speed_7d):
    '''
    rfile[output]: the name of the html file to write 
    #FFFF00 yellow
    #33FF38 green
    #FFA500 orange
    #FF0000 red
    #FFFFFF white
    '''
    #Check final data for project
    end_proj = harmonconf.yaml["PROJECTS"][project]["END_DATE"]
    #Set this by default to white and no remark
    extra_remark = ""
    bgcolor = "#FFFFFF"
    if speed_7d == 0.0:
        #print(f"{stream} not active for past week")
        extra_remark = "Inactive the past week"
        bgcolor = "#FFFF00"
    if speed_7d  == 0.0 and speed_30d  < 1.0: # and stream != "carra_pan":
        #print(f"{stream} not active for past week")
        extra_remark = "Currently suspended"
        bgcolor = "#FFFF00"
    #if speed_30d < 3.0 and stream != "carra_pan":
    #    extra_remark = "Delayed due to production issues"
    #    bgcolor = "#FFFF00"
    if speed_30d == 0.0 and speed_7d == 0.0:
        str_comp,str_end = stream_completed(project,stream)
        if str_comp: 
            extra_remark = "Stream finished"
            bgcolor = "#33FF38"
        else:
            extra_remark = "Currently suspended"
            bgcolor = "#FFFF00"
    #ALWAYS CHECK if the stream is finished!!!
    str_comp,str_end = stream_completed(project,stream)
    if str_comp: 
        extra_remark = "Stream finished"
        bgcolor = "#33FF38"

    #The orange color should always superseed all others
    if (TOAstream.isnumeric() 
        and int(TOAstream[0:8]) > end_proj 
        and not any(x in extra_remark for x in ["suspended","finished"])):
        #and "suspended" not in extra_remark:
        print(f"WARNING: {stream} is ending after project end {end_proj}")
        extra_remark = f"Completion date after project end ({end_proj})"
        bgcolor = "#FFA500"
    #elif TOAstream == "Inactive":
    #    extra_remark = "Currently suspended"
    #    bgcolor = "#FFFF00"
    #    print(f"WARNING: {stream} is inactive")
    #else:
    #    extra_remark = ""
    #    bgcolor = "#FFFFFF"
    extra_flag=""
    progCARRA=progress_bar("CARRA")
    progPA=progress_bar("PANARCTIC")
    exists = os.path.isfile(rfile)
    header='<!doctyle html><html><head><title>Estimated completion dates for all streams</title></head><body>'
    maintitle='<h1>Estimated completion dates and simulation speeds</h1>'
    description='<p>Stream (production) completion date based on 30d (7d) moving avg speed</p>'
    #extranote='<p> (*) Stream active for less than 5 days in the current month (completion date based on 7d avg). (**) Stream completed. (***) Stream suspended. Completion date undetermined. </p>'
    explanation='<p><a href="https://docs.google.com/document/d/1zDKAxuEXYnWWYzT3SZnZwY5WW8nodUMeYRjAGz46coc/edit#">Details of the calculation of the completion dates</a></p>'
    lastchanged='<p>Last update: '+datetime.now().strftime('%Y/%m/%d %H:%M:%S')+' UTC</p>'
    completed='<p>Total progress CARRA: '+str(progCARRA)+'% <a href="https://hirlam.org/portal/CARRA/Progress/GanntChart_CARRA.html"> (see detailed progress meter in Gantt chart) </a></p>'
    compPA='<p>Total progress PANARCTIC: '+str(progPA)+'% </p>'
    waittimes='<p><a href="https://hirlam.org/portal/CARRA/Progress/waiting_times.html"> Queue waiting times for all streams</a></p>'
    marsarxiv='<p><a href="https://hirlam.org/portal/CARRA/Progress/mars_status.html"> MARS archive throughput </a></p>'
    body = '<html><table border="1"><tr><th>expId</th><th>Current assimilated date</th><th>End date of stream </th><th> Completion date</th><th> Avg. throughput (7d) </th> <th>Avg. throughput (30d) </th> <th> Remarks</tr>'
    footer='</table></body></html>'
    cdate=get_DTG(project,stream)
    end_date = harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["END_DATE"]
    wspeed = "("+str(speed_7d)+") days/day"
    mspeed = "("+str(speed_30d)+") days/day"
    EXTRA="TEST"
    if exists==True:
        with open(rfile, 'a') as f:
            f.write(f'<tr bgcolor="{bgcolor}"><td>{stream}</td><td>{cdate}</td><td>{end_date}</td><td>{TOAstream}</td><td>{wspeed}</td><td>{mspeed}</td><td>{extra_remark}</tr>\n')
    else:
        with open(rfile, 'w') as f:
            f.write(header+"\n")
            f.write(maintitle+"\n")
            f.write(description+"\n")
            #f.write(extranote+"\n")
            f.write(explanation+"\n")
            f.write(lastchanged+"\n")
            f.write(completed+"\n")
            f.write(compPA+"\n")
            f.write(waittimes+"\n")
            f.write(marsarxiv+"\n")
            f.write(body+"\n")
            f.write(f'<tr bgcolor="{bgcolor}"><td>{stream}</td><td>{cdate}</td><td>{end_date}</td><td>{TOAstream}</td><td>{wspeed}</td><td>{mspeed}</td><td>{extra_remark}</tr>\n')



def html_table_danra(rfile,project,stream,TOAstream,expectedDTG,speed_7d,speed_30d,streams_per_user):
    '''
    Prints a the html file using the DANRA format

    rfile[output]: the name of the html file to write 
    '''
    prog2021=progress_bar()
    progress2021='<p>Total progress: '+str(prog2021)+'% for the scheduled delivery in 2021</p>'

    gantt='<p> <a href="https://hirlam.org/portal/DKREA/Progress/GanttChart_DKREA.html"> Gantt chart showing detailed progress meter </a></p>'
    goback='<p> <a href="https://hirlam.org/portal/oprint/WebgraF/DKREA/"> Return to DKREA monitoring interface </a></p>'

    rdir='/home/ms/dk/'+streams_per_user[stream]+'/hm_home' #this is where to look for progress.log
    dtgbeg,dtgend = stream_periods(project,stream)
    pfile=os.path.join(*[rdir,stream,'progress.log'])
    exists = os.path.isfile(rfile)
    header='<!doctyle html><html><head><title>Estimated completion dates for all streams</title></head><body>'
    maintitle='<h1>Estimated completion dates and simulation speeds</h1>'
    description='<p>Stream completion date based on 30d (7d for newer runs). Set to 0 for simulations with < 7d </p>'
    lastchanged='<p>Last update: '+datetime.now().strftime('%Y/%m/%d %H:%M:%S')+' UTC</p>'

    body = '<html><table border="1"><tr><th>expId</th><th>Current assimilated date</th><th>End date of stream </th><th> Completion date</th><th> Avg. throughput (7d) </th> <th>Avg. throughput (30d) </th> </tr>'
    footer='</table></body></html>'
    with open(pfile, 'r') as f:
        lines=f.readlines()
        cdate=re.search('DTG=(.*) export',lines[0]).group(1)
    if exists==True:
        with open(rfile, 'a') as f:
            f.write('<tr><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td></tr>\n'.format(stream,cdate,dtgend,TOAstream,"("+str(speed_7d)+") days/day","("+str(speed_30d)+") days/day"))
            #f.write('<tr><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td></tr>\n'.format(stream,cdate,dtgend,TOAstream,expectedDTG, "("+str(speed_7d)+") days/day","("+str(speed_30d)+") days/day"))
    else:        
        with open(rfile, 'w') as f:
            f.write(header+"\n")
            f.write(maintitle+"\n")
            f.write(progress2021+"\n")
            f.write(gantt+"\n")
            f.write(goback+"\n")
            f.write(description+"\n")
            f.write(lastchanged+"\n")
            f.write(body+"\n")
            #f.write('<tr><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td></tr>\n'.format(stream,cdate,dtgend,TOAstream,expectedDTG, "("+str(speed_7d)+") days/day","("+str(speed_30d)+") days/day"))
            f.write('<tr><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td></tr>\n'.format(stream,cdate,dtgend,TOAstream,"("+str(speed_7d)+") days/day","("+str(speed_30d)+") days/day"))

def stream_periods(project,stream):
    '''
    Periods to run for each stream
    '''
    conf = harmonconf.yaml["PROJECTS"][project]["STREAMS"]
    beg_date = str(conf[stream]["BEG_DATE"])
    end_date = str(conf[stream]["END_DATE"])
    return beg_date,end_date


def DTG_bydate(sim,bydate,csd,avg_speed):
    '''
    Determine which simulation date we will reach for a stream by a given date (bydate)
		'''
    today=datetime.today()
    final_date = datetime.strptime(bydate,'%Y%m%d')
    togo = (final_date - today).days
    #print(f"Days to go until {final_date}: {togo}")
    if avg_speed != 0:
        days_to_simulate = avg_speed*togo # days to simulate at current speed
        DTG_expected = datetime.strptime(csd,'%Y%m%d%H') + timedelta(days=days_to_simulate)
        DTG = datetime.strftime(DTG_expected,'%Y%m%d%H')
        end_date = datetime.strftime(final_date,'%Y%m%d')
        if DTG_expected > final_date:
            print(f"WARNING: final date by {bydate} is in the future! Final date expected: {DTG}")
            print(f"Setting an upper limit equal to {bydate}-30 days")
            DTG_expected = final_date - timedelta(days=30)
            DTG = datetime.strftime(DTG_expected,'%Y%m%d%H')
        print(f"Estimated DTG by {end_date}: {DTG}")
    return DTG

def TOA_stream(project,stream,csd,cur_avg_speed,beg_date,end_date):
    '''
    Calculation of the TOA for the whole stream
    '''
    today=datetime.today()
    #stoday = datetime.strftime(datetime.today,'%Y%m%d')
    total = datetime.strptime(end_date,'%Y%m%d%H')-datetime.strptime(beg_date,'%Y%m%d%H')
    time_to_simulate=total.days
    total = datetime.strptime(csd,'%Y%m%d%H') - datetime.strptime(beg_date,'%Y%m%d%H')
    time_simulated=total.days #number of simulated days

    still_todo = time_to_simulate - time_simulated
    #still_todo_again = datetime.strptime(end_date,'%Y%m%d%H')-datetime.strptime(csd,'%Y%m%d%H')
    #print("Calc again time todo (current: %s) %d"%(csd,still_todo_again.days))
    if cur_avg_speed != 0:
        togo=still_todo/cur_avg_speed #real amount of days to go
        print("simulated time still todo %d"%still_todo)
        print("time to simulate %g"%togo)
        TOA = today + timedelta(days=togo) # the date when all should be finished at current avg speed
        TOA_print = datetime.strftime(TOA,"%Y%m%d%H")
    else:
        #check if stream is unactive because if finished
        str_comp,str_end = stream_completed(project,stream)
        if str_comp:
            print(f"{stream} ended on {str_end}")
            TOA_print=f"Finished on {str_end}"
        else:
            TOA_print="Inactive"

    return TOA_print


def get_DTG(project,stream):
    '''
    get current DTG
    '''
    user = harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["USER"]
    local_path = "/home/ms/dk/"+user+"/hm_home"
    stream_path = os.path.join(local_path,stream)
    pfile=os.path.join(stream_path,'progress.log')

    with open(pfile, 'r') as f:
        lines=f.readlines()
        cdate=re.search('DTG=(.*) export',lines[0]).group(1)
    return cdate

def stream_completed(project,stream):
    '''
    Check a stream to see if completed,
    and return completion date
    '''
    stream_is_finished = False
    completion_date = None
    end_date = harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["END_DATE"]
    cdate=int(get_DTG(project,stream))
    user = harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["USER"]
    local_path = "/home/ms/dk/"+user+"/hm_home"
    stream_path = os.path.join(local_path,stream)
    pfile=os.path.join(stream_path,'progress.log')
    if cdate >= end_date:
         mtime=os.path.getmtime(pfile)
         completion_date=time.strftime('%Y%m%d',time.localtime(mtime))
         stream_is_finished = True
         print(f"from stream_completed: {stream} finished on {completion_date}")
    return stream_is_finished,completion_date

def production_start(project,stream):
    '''
    Returns the date when the production started, and a boolean if production was started
    (that is, when the warmup year was completed)
    '''
    stream_prod = False
    prod_start = None
    beg_prod = harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["BEG_PROD"]
    csd = get_DTG(project,stream)
    dtgcur=datetime.strptime(csd,'%Y%m%d%H')
    dtgprod=datetime.strptime(beg_prod,'%Y%m%d%H')
    if dtgcur >= dtgprod:
        conn=sqlite3.connect(dbase)
        sql_command = "SELECT * FROM daily_logs"
        data=pd.read_sql(sql_command, conn)
        search_string = "/".join([csd[0:4],csd[4:6],csd[6:8]])
        get_date=data.logfile.str.contains(search_string)
        if get_date.empty:
            print(f"Date {search_string} not found in record for {stream}")
            sys.exit(1)
        prod_start = get_date["yyymmdd"]
        stream_prod = True
        
    return stream_prod,prod_start


def sim_speed_total(use_streams,project,cur_avg_speed):
    '''
    Calculates real and ideal speed (target and min speed)
    for the whole project (all streams) or a selected subset
    Takes into account the warmup period
    Returns min and target speed based on the length of each stream
    Also provides an estimate of the TOA for the end of the warm-up year, but this can be changed
    based on the current 1-week moving-average.
    '''
    buffer_days = BUFFER_DAYS
    from dateutil import relativedelta
    sim_total = 0
    if use_streams is None:
        streams = [key for key in harmonconf.yaml["PROJECTS"][project]["STREAMS"].keys()]
    elif "," in use_streams:
        streams = use_streams.split(",")

    for stream in streams:
        csd = get_DTG(project,stream)
        beg_date = str(harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["BEG_DATE"])
        end_date = str(harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["END_DATE"])
        beg_prod = str(harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["BEG_PROD"])
        warmupdays2sim = (datetime.strptime(end_date,'%Y%m%d%H') - datetime.strptime(beg_prod,"%Y%m%d%H")).days
        sim_total += (datetime.strptime(end_date,'%Y%m%d%H') - datetime.strptime(csd,"%Y%m%d%H")).days
    print(f"Total number of days simulated so far for all streams {sim_total}")
    today=datetime.today()
    #These are dates for the whole project
    beg_proj = str(harmonconf.yaml["PROJECTS"][project]["BEG_DATE"])
    end_proj = str(harmonconf.yaml["PROJECTS"][project]["END_DATE"])
    #total number of project days
    pdays = (datetime.strptime(end_proj,'%Y%m%d')-datetime.strptime(beg_proj,'%Y%m%d')).days
    #Number of warmup days
    #total number of days to simulate for this stream
    pdays_passed = (today - datetime.strptime(beg_proj,'%Y%m%d')).days#number of days already spent
    pdays_left = pdays - pdays_passed
    print(f"Project days remaining {pdays_left}")
    
    min_speed = sim_total/pdays_left #min speed we need to have to end the project
    if pdays_left > buffer_days:
        target_speed = sim_total/(pdays_left-buffer_days) #same as above, but accounting for 2 months buffer
    elif pdays_left >= 10:
        target_speed = sim_total/(pdays_left-10) #same as above, but accounting for 10 days
    else:
        print("Simulation has been running for less than 10 days, target_speed is the same as min_speed")
        target_speed = sim_total/pdays_left
    print(f"min and target speed: {min_speed},{target_speed}")
    #Simulated days
    #csd = get_DTG(stream)
    #days_to_simulate=datetime.timedelta(days=warmupdays2sim)
    #dtgcur=datetime.strptime(csd,'%Y%m%d%H')
    #time_simulated = datetime.strptime(csd,'%Y%m%d%H') - datetime.strptime(beg_date,'%Y%m%d%H')
    #time_to_simulate = days_to_simulate.days - time_simulated.days
    #print("Remaining days in %s stream %d for warmup phase: %d (last DTG read: %s)"%(domain,slabel,time_to_simulate,dtgcur))
    #if cur_avg_speed != 0 and not stream_completed(stream):
    #    togo=time_to_simulate/cur_avg_speed #days to go for warmup at current speed
    #    if time_to_simulate < 0:
    #        print("Warmup year already completed for %s stream %d"%(domain,slabel))
    #        togo = 0
    #        TOAproduction = production_start(stream)
    #    else:
    #        TOAproduction = today + datetime.timedelta(days=togo) # the date when all should be finished at current avg speed and the production should start
    #elif completed_streams[streamname]:
    #    TOAproduction=datetime.strptime(completed_streams_dates[streamname],'%Y%m%d')
    #    min_speed = 0.0
    #    target_speed = 0.0
    #else:
    #    TOAproduction=datetime(2099,12,31,0,0,0) #"9999"
    return min_speed, target_speed #, TOAproduction

def days2go(project,stream):
    '''
    Calculation of days to go for a stream
    '''
    beg_date = str(harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["BEG_DATE"])
    end_date = str(harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["END_DATE"])
    csd = get_DTG(project,stream)
    today=datetime.today()
    total = datetime.strptime(end_date,'%Y%m%d%H')-datetime.strptime(beg_date,'%Y%m%d%H')
    total_simulated = datetime.strptime(csd,'%Y%m%d%H') - datetime.strptime(beg_date,'%Y%m%d%H')
    days_togo = total.days - total_simulated.days
    print(f"Total number of days to simulate in {stream} = {total.days}")
    print(f"Still to go in {stream} = {days_togo}")
    print(f"Progress so far: {total.days-days_togo}")
    return days_togo


def speed_stream(project,stream):
    '''
    returns the min and target speed
    '''
    
    #update project days to go as of today
    today=datetime.today()
    end_real = str(harmonconf.yaml["PROJECTS"][project]["END_DATE"])
    finalday_real = datetime.strptime(end_real,'%Y%m%d')

    end_real_date = datetime.strptime(end_real,"%Y%m%d")
    finalday_ideal = end_real_date - timedelta(days=BUFFER_DAYS)  

    projdays_ideal = (finalday_ideal - today).days
    projdays_real = (finalday_real - today).days

    #update days to go in current stream
    togo_sim = days2go(project,stream)
    print(f"Days still to simulated in {stream}:{togo_sim}")
    print(f"Days remaining in project: {finalday_real}")
    min_speed = togo_sim/projdays_real #same as above, but accounting for 2 months buffer
    target_speed = togo_sim/projdays_ideal #total number of project days remaining from 20190626
    return min_speed,target_speed






#check the timestamps of the log files under 
# wrkDir=/scratch/ms/dk/nhx/hm_home/
# create files with the latest timestamp for the day.
# Update the final files
import os
import time
from collections import OrderedDict
import pandas as pd
from datetime import datetime
from datetime import timedelta
import re
import sys
import subprocess
import sqlite3

from hartools.harmon.config import harmonconf as harmonconf
from hartools.harmon.count_days_ecfs import count_days as cdays

#Class to store all time stamps for a given stream
class log_timestamps(object):
    def __init__(self, project=None, streams=None, date=None, dbase=None):
        self.project = project
        self.streams = streams # a dict with keys STREAM and USER and a list in each
        self.date = date # date, in format YYYYMMDD
        self.dbase = dbase
        #The date needs to match the format in els!
        #Note the extra space with one digit Jun  8
        # instead of Jun 10
        # This removes the extra zero: datetime.strftime(date_conv,"%b %-d")
        # This adds the extra space: datetime.strftime(date_conv,"%b  %-d")
        if self.date is None:
            today=datetime.today()
            sdigit = datetime.strftime(today,"%-d")
            if len(sdigit) == 1: self.date = datetime.strftime(today,"%b  %-d")
            if len(sdigit) > 1: self.date = datetime.strftime(today,"%b %-d")
        else:
            date_conv = datetime.strptime(self.date,"%Y%m%d")
            sdigit = datetime.strftime(date_conv,"%-d")
            if len(sdigit) == 1: self.date = datetime.strftime(date_conv,"%b  %-d")
            if len(sdigit) > 1: self.date = datetime.strftime(date_conv,"%b %-d")
        self.ts_old = self._read_sql(self.dbase)
        self.counter, self.ts_new = self._new_timestamps(self.project,self.streams,self.date)
        self.ts = self._clean_timestamps(self.ts_old,self.ts_new)

    def _new_timestamps(self,project,streams,date):
        counter,timestamps = cdays(project,streams,date)
        return counter, timestamps

    def save_data(self,dbase):
        conn=sqlite3.connect(dbase)
        self.ts.to_sql("daily_logs",conn, if_exists="replace", index=False)

    def _clean_timestamps(self,df_old,df_new):
        '''
        Merge the old and new data sets, getting
        rid of any duplicates
        @return cleaned data frame
        '''
        dfs=[df_old, df_new]
        merge=pd.concat(dfs,sort=False)
        merge.sort_values('timestamp',inplace=True)
        merge.drop_duplicates(subset=['stream','timestamp'], keep='first',inplace=True)
        merge.sort_values('stream',inplace=True)
        merge.reset_index(drop=True,inplace=True)
        return merge

    def _read_sql(self,sql_file):
        '''
        create table if it does not exist.
        @return dataframe with time stamps
        '''
        if not os.path.isfile(sql_file):
            self.create_table(sql_file)
        conn=sqlite3.connect(sql_file)
        sql_command = "SELECT * FROM daily_logs"
        ts = pd.read_sql(sql_command, conn)
        return ts

    def create_table(self,sql_file,table="daily_logs"):
        """ 
        create the sql table
        """
        create_table_sql   = """CREATE TABLE IF NOT EXISTS """+table+"""
                              ( stream TEXT PRIMARY KEY, 
                                logfile TEXT NOT NULL, 
                                timestamp TEXT NOT NULL, 
    			        yyyymmdd TEXT NOT NULL,
                                simdate TEXT NOT NULL )"""
    
        conn=sqlite3.connect(sql_file)
        try:
            c = conn.cursor()
            c.execute(create_table_sql)
        except Error as e:
            print(e)

    def speed_stream(self,stream,winlen):
        '''
        print stream speed for a particular window length
        '''
        #group by stream and real date
        gp=self.ts.groupby(['yyyymmdd','stream'])
        #select the groups for the current date        
        #today=datetime.strftime(datetime.now(),'%Y/%m/%d')
        days=[datetime.today() - timedelta(days=x) for x in range(0,winlen)]
        #Loop through the logfiles for this date and the past days
        #in the winlen window. Extract only the DTG part from logfile name
        count_sim_days=0
        for day in days:
            #select the indices that I need to access for this day
            key=(datetime.strftime(day,'%Y%m%d'), stream)
            if key in gp.groups:
                idxs=gp.groups[key].tolist()
                sim_dates=[]
                for ix in idxs:
                    get_log=self.ts['logfile'][ix]
                    sel_file=os.path.split(get_log)[1]
                    sel_date=os.path.splitext(sel_file)[0].split('_')[-1]
                    yyyymmdd=sel_date[0:8]
                    if yyyymmdd not in sim_dates:
                        sim_dates.append(yyyymmdd)
                count_sim_days=len(sim_dates)+count_sim_days
        if count_sim_days > 0:
            sim_speed = count_sim_days/winlen
        else:
            sim_speed = 0.0

        return sim_speed


if __name__ == "__main__":
    #TESTS: not to be used to collect data!
    project="CARRA"
    streams= [key for key in harmonconf.yaml["PROJECTS"][project]["STREAMS"].keys()]
    #print(f"Collecting data for {streams}")        
    #dbase = harmonconf.yaml["ARXIV"]["DBASE"]
    #timestamps=log_timestamps(project=project,streams=streams,dbase=dbase)
    #timestamps.save_data(dbase=dbase)
    #print(f"Saving data to {outdir}")
    #ts_streams.save_data(outdir=outdir)



'''
check the timestamps of the log files under 
directly in ecfs
It will select only the directories containing
logfiles.tar, since this file always has to be
there when a YYYY/MM/DD/HH is completed
'''
import os
import time
from collections import OrderedDict
import pandas as pd
import re
import sys
import subprocess
from hartools.harmon.config import harmonconf as harmonconf
from datetime import datetime
import time
from functools import wraps
import hartools.harmon.progress_utils as pu

def timing(f):
    '''
    Decorator for timing functions
    Usage: include the timing decorator
    before the function defintion as  below

    @timing
    def function(a):
        pass

    @return timing of the function in seconds
    '''
    @wraps(f)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = f(*args, **kwargs)
        end = time.time()
        #print('function:%r took: %2.2f sec' % (f.__name__,  end - start))
        print('func:%r args:[%r, %r] took: %2.4f sec'%(f.__name__, args, kwargs, end-start))
        return result
    return wrapper

def els_cmd(edir,com="els -l"):
    '''
    Check a particular file. IT must return only one element, otherwise it failed   
    '''
    cmd = com+" "+edir
    try:
        ret=subprocess.check_output(cmd,shell=True)

    except subprocess.CalledProcessError as err:
        print(f"Error in call to subprocess {err}")
        print(f"{cmd} failed")
        rubbish = ["None"] # dummy array with 2 elements
        return rubbish
    ret = ret.decode('utf-8')
    #split listing in lines
    elsdir = ret.split("\n")
    elsdir.remove("") #remove empty elements
    return elsdir

def get_month(elslist,stream,today):
    '''
    Sel month(s) based on the last modified date
    @return: list of months
    '''
    #The split cleans the extra spaces, date and month appear in 
    # places 6 and 7
    this_year = datetime.strftime(datetime.now(),"%Y")
    date_strings = [d.split()[5]+" "+d.split()[6] for d in elslist]
    sel_m = [l.split()[-1] for l in elslist if l.split()[-1].isdigit()]

    if len(sel_m) != 1:
        #print(f"WARNING: more than one directory available on month: {sel_m}")
        sel_month = [d.split()[-1] for d in elslist if today in d]
        if len(sel_month) != 0:
            print(f"Last month modified on {today}: {sel_month}")
        else: #print dates of year directories for control
            lastmod = [l.split()[-1]+" "+l.split()[5]+" "+l.split()[6] for l in elslist if l.split()[-1]]
            print(f"DEBUG: last months modified (month and date): {lastmod}")
       
    
    return sel_m

def get_year(elslist,stream,today):
    '''
    Sel years in directory
    '''
    #The split cleans the extra spaces, date and month appear in 
    # places 6 and 7
    this_year = datetime.strftime(datetime.now(),"%Y")
    date_strings = [d.split()[5]+" "+d.split()[6] for d in elslist]
    sel_y = [l.split()[-1] for l in elslist if l.split()[-1].isdigit()]

    if len(sel_y) != 1:
        #print(f"WARNING: more than one directory available on year: {sel_y}")
        sel_year = [d.split()[-1] for d in elslist if today in d]
        if len(sel_year) != 0:
            print(f"Last dir modified on {today}: {sel_year}")
        else: #print dates of year directories for control
            lastmod = [l.split()[-1]+" "+l.split()[5]+" "+l.split()[6] for l in elslist if l.split()[-1]]
            print(f"DEBUG: last years modified: {lastmod}")
    #sel_y = max(sel_y)
    return sel_y

def get_days(month_ls,today,edir):
    '''
    Search for the days which might contain data
    on today date
    '''
    days = [d.split()[-1] for d in month_ls]
    sel_days=[]
    for day in days:
        ddir = os.path.join(edir,day)
        ls_day = els_cmd(ddir)
        lookuphours = [l for l in ls_day if today in l]
        #debug message
        #if len(lookuphours) != 0: print(f"Found {lookuphours} in {day}")
        if len(lookuphours) != 0: sel_days.append(day)
    return sel_days

def get_dh(elslist,today):
    '''
    Sel days and hours
    '''
    #The split cleans the extra spaces. Date and month appear in 
    # places 6 and 7. The directory in last place
    sel_dir = [d.split()[-1] for d in elslist if today in d]
    #select the year/month, it should be only one!
    sel_dh = [l for l in sel_dir if l.isdigit()]
    return sel_dh
#@timing
#This function is only for testing. Note hardcoded project below
def alldates_stream(stream):
    '''
    Go down all year/month/days directories
    for an stream
    This is only to collect the total number of 
    timestamps for a stream. It can take up to 3 h running in
    an slurm script to do the els commands for a whole stream.
    @return a df with all timestamps info
    '''
    project="DANRA"
    user = harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["USER"]
    if os.path.isfile(stream+".csv.gz"):
        timestamps = pd.read_csv(stream+".csv.gz",compression="gzip")
        timestamps.drop(columns=["id"],inplace=True)
        return timestamps
       
        #read the data from this file
    #timestamps.to_csv(stream+".csv",sep=",")
    print(f"Doing {stream}")
    estream  = os.path.join("ec:/"+user+"/harmonie",stream)
    this_year = datetime.strftime(datetime.now(),"%Y") # current year
    logs_path=[]
    elsfiles=els_cmd(estream)
    years = [x.split()[-1] for x in elsfiles if x.split()[-1].isdigit()]
    for year in years:
        months = els_cmd(os.path.join(estream,year),"els")
        for month in months:
            days = els_cmd(os.path.join(estream,year,month),"els") 
            for day in days:
                hours = els_cmd(os.path.join(estream,year,month,day),"els") 
                for hour in hours:
                    logfile = os.path.join(estream,year,month,day,hour,"logfiles.tar")
                    test = els_cmd(logfile,"els") 
                    if len(test) != 1 or "None" in test:
                        print(f"{logfile} not found")
                    else:
                        logs_path.append(logfile)
    collect_all =OrderedDict()
    collect_all["stream"] = []
    collect_all["date"] = []
    collect_all["path"] = []
    collect_all["simdate"] = []
    collect_all["timestamp"] = []
    #print("All files found")
    for log in logs_path:
        collect_all["stream"].append(stream)
        logls = els_cmd(log,"els -l")
        if not ":"  in logls[0].split()[7]:
            datestring = " ".join([logls[0].split()[5],logls[0].split()[6],logls[0].split()[7]])
            tstamp = datetime.strptime(datestring,"%b %d %Y")
        else:
            datestring = " ".join([this_year,logls[0].split()[5],logls[0].split()[6],logls[0].split()[7]])
            tstamp = datetime.strptime(datestring,"%Y %b %d %H:%M")
        drf = datetime.strftime(tstamp,"%Y/%m/%d")
        rftstamp = datetime.strftime(tstamp,"%Y-%m-%d %H:%M:%S")
        collect_all["timestamp"].append(rftstamp)
        collect_all["date"].append(drf)
        log_split = log.replace("ec:","").split("/")
        year,month,day = log_split[4],log_split[5],log_split[6]
        collect_all["simdate"].append(os.path.join(year,month,day))
        collect_all["path"].append(log)
        #print(l)
    dates_df = pd.DataFrame(collect_all)
    dates_df.drop_duplicates(subset="simdate",keep="last",inplace=True)
    timestamps = pd.DataFrame({"stream":collect_all["stream"],
                               "logfile":collect_all["path"],
                               "timestamp":collect_all["timestamp"],
                               "yyyymmdd":collect_all["date"],
                               "simdate":collect_all["simdate"]})
    #This one is just for safekeeping in case data is somehow lost
    #in the merge with the old data as happened once
    #Moved to collect_all for the moment
    #timestamps.to_csv(stream+".csv",sep=",")
    return timestamps

def getdate_els(elslist,project,stream,today):
    '''
    Collect all dates for a given stream.
    It is supposed to be used only with currently running streams
    '''

    # Clean the els listing and extract dates for each file
    this_year = datetime.strftime(datetime.now(),"%Y")
    #go through year and month
    years = get_year(elslist,stream,today)

    #Only check currrent year and last year in list above (in case this happened on same day)
    csd = pu.get_DTG(project,stream)
    csd_year = csd[0:4]
    years = [years[-2], csd_year]
    #TODO: need to add hear an extra loop for month, since there
    #can be a change of month here on the same day!
    # Include a loop??
    collect_all =OrderedDict()
    collect_all["stream"] = []
    collect_all["date"] = []
    collect_all["path"] = []
    collect_all["simdate"] = []
    collect_all["timestamp"] = []
    user = harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["USER"]
    for year in years:
        edir  = os.path.join("ec:/"+user+"/harmonie",stream,year)
        print(f"Checking {edir}")
        ls_year = els_cmd(edir)
        sel_month = get_month(ls_year,stream,today)
        for month in sel_month:
            edir  = os.path.join("ec:/"+user+"/harmonie",stream,year,month)
            print(f"Checking {edir}")
            ls_month = els_cmd(edir)
            #print(ls_month)
            #go through days and hours
            #if month == "05" and year == "1995":
            sel_day = get_days(ls_month,today,edir)
            #else:
            #    sel_day = get_dh(ls_month,today)
            for day in sel_day:
                edir  = os.path.join("ec:/"+user+"/harmonie",stream,year,month,day)
                #print(edir)
                ls_day = els_cmd(edir)
                sel_hour = get_dh(ls_day,today)
                print(f"Hours in day {day}: {sel_hour}")
                for hour in sel_hour:
                    edir  = os.path.join("ec:/"+user+"/harmonie",stream,year,month,day,hour)
                    #print(f"check day: {edir}")
                    ls_hour = els_cmd(edir)
                    log = [l for l in ls_hour if "logfiles.tar" in l]
                    if len(log) != 0:
                        log = log[0]
                        # The datestring can also be of the form
                        # Mar 24  2020 for past years. It will only contain
                        # form mmm dd HH:MM for current year
                        if not ":"  in log.split()[7]:
                            datestring = " ".join([log.split()[5],log.split()[6],log.split()[7]])
                            tstamp = datetime.strptime(datestring,"%b %d %Y")
                        else:
                            datestring = " ".join([this_year,log.split()[5],log.split()[6],log.split()[7]])
                            tstamp = datetime.strptime(datestring,"%Y %b %d %H:%M")
                        drf = datetime.strftime(tstamp,"%Y/%m/%d")
                        rftstamp = datetime.strftime(tstamp,"%Y-%m-%d %H:%M:%S")
                        collect_all["date"].append(drf)
                        collect_all["timestamp"].append(rftstamp)
                        collect_all["stream"].append(stream)
                        collect_all["simdate"].append(os.path.join(year,month,day))
                        addpath = os.path.join(edir,"logfiles.tar")
                        collect_all["path"].append(addpath)
                        print(f"Found {addpath} with timestamp: {datestring}")
    #Now select only the last item in a given day
    dates_df = pd.DataFrame(collect_all)
    dates_df.drop_duplicates(subset="simdate",keep="last",inplace=True)
    timestamps = pd.DataFrame({"stream":collect_all["stream"],
                               "logfile":collect_all["path"],
                               "timestamp":collect_all["timestamp"],
                               "yyyymmdd":collect_all["date"],
                               "simdate":collect_all["simdate"]})
    return dates_df["date"], timestamps
     

def count_days(project,streams,today):
    '''
    Count latest files with els in the harmonie directories
    @today: string format, ie Jan 12
    '''
    counter=OrderedDict()
    counter["ndays"] = []
    counter["stream"] = []
    ts_streams = []
    for stream in streams: 
        print(f"Cheking {stream} on {today}")
        user = harmonconf.yaml["PROJECTS"][project]["STREAMS"][stream]["USER"]
        edir  = "ec:/"+user+"/harmonie/"+stream
        elsfiles=els_cmd(edir)
        dates,tstamps = getdate_els(elsfiles,project,stream,today)
        ndays = dates.shape[0]
        print(f"Simdays completed for {stream} on {today}: {ndays}")
        counter["ndays"].append(ndays)
        counter["stream"].append(stream)
        ts_streams.append(tstamps)
    merge=pd.concat(ts_streams,sort=False)
    merge.sort_values('timestamp',inplace=True)
    merge.reset_index(drop=True,inplace=True)
    return counter,merge
if __name__=="__main__":
    #TESTS:
    project="CARRA"
    streams = [key for key in harmonconf.yaml["PROJECTS"][project]["STREAMS"].keys()]
    #streams = ["carra_NE_4","carra_NE_5","carra_NE_6",
    #           "carra_IGB_4","carra_IGB_5","carra_IGB_6"]
    #provide input as date of form YYYYMMDD
    #Otherwise it will use today. It will convert to format
    # Month (3 letters) day, to match what els prints
    if len(sys.argv) == 2:
        today_in = sys.argv[1]
        #The date format needs to match what is printed by ls -l
        #Extra space for 1 digit: June  8 (cf Jun 10)
        date = datetime.strptime(today_in,"%Y%m%d")
        sdigit = datetime.strftime(date,"%-d")
        if len(sdigit) == 1: today = datetime.strftime(date,"%b  %-d")
        if len(sdigit) > 1:  today = datetime.strftime(date,"%b %-d")
    else:
        sdigit = datetime.strftime(datetime.now(),"%-d")
        if len(sdigit) == 1: today = datetime.strftime(datetime.now(),"%b  %-d")
        if len(sdigit) > 1:  today = datetime.strftime(datetime.now(),"%b %-d")
    counter,streams_ts = count_days(project,streams,today)
    print(f"Summary of days simulated on {today}")
    print(counter)
    
    print(f"Extraction based on counting numbers in dataframe")
    for stream in streams:
        count = streams_ts[streams_ts.stream == stream].drop_duplicates(subset="simdate",keep="last").shape[0]
        print(f"{count} days on {today} for {stream}")



"""Main module.
   Including here Config classes
"""

#from typing import Set
#
#from pydantic import (
#    BaseModel,
#    BaseSettings,
#    PyObject,
#    Field,
#)
#
#
#class SubModel(BaseModel):
#    foo = 'bar'
#    apple = 1
#
#
#class Settings(BaseSettings):
#    auth_key: str
#    api_key: str = Field(..., env='MY_CONFIG')
#    more_settings: SubModel = SubModel()
#
#    class Config:
#        env_prefix = 'my_prefix_'  # defaults to no prefix, i.e. ""
#        fields = {
#            'auth_key': {
#                'env': 'my_auth_key',
#            },
#        }
#

#print(Settings().dict())
"""
{
    'auth_key': 'xxx',
    'api_key': 'xxx',
    'redis_dsn': RedisDsn('redis://user:pass@localhost:6379/1',
scheme='redis', user='user', password='pass', host='localhost',
host_type='int_domain', port='6379', path='/1'),
    'pg_dsn': PostgresDsn('postgres://user:pass@localhost:5432/foobar',
scheme='postgres', user='user', password='pass', host='localhost',
host_type='int_domain', port='5432', path='/foobar'),
    'amqp_dsn': AmqpDsn('amqp://user:pass@localhost:5672/', scheme='amqp',
user='user', password='pass', host='localhost', host_type='int_domain',
port='5672', path='/'),
    'special_function': <built-in function cos>,
    'domains': set(),
    'more_settings': {'foo': 'bar', 'apple': 1},
}
"""

